/****************************************************************************
*
*     Copyright (c) 2003 Dave Hylands
*           All Rights Reserved
*
*   Permission is granted to any individual or institution to use, copy, or
*   redistribute this software so long as it is not sold for profit, and that
*   this copyright notice is retained.
*
****************************************************************************/
/**
*
*   @file   Exception.cpp
*
*   @brief  Implements helper routines when working with exceptions
*
****************************************************************************/

// ---- Include Files -------------------------------------------------------

#include <stdio.h>
#include "Exception.h"
#include <68332/isr.h>
#include <68332/qsm_polled.h>

// ---- Public Variables ----------------------------------------------------
// ---- Private Constants and Types -----------------------------------------

// ---- Private Variables ---------------------------------------------------
// ---- Private Function Prototypes -----------------------------------------

// ---- Functions -----------------------------------------------------------

static void ExceptionHandler( unsigned int vectorNum, unsigned long pc );

//***************************************************************************
/**
*   Reports that an unexpected exception has occurred
*/

// static
void ExceptionHandler( unsigned int vectorNum, unsigned long pc )
{
    char    buf[ 60 ];
    char   *s = buf;

    set_int_level( 7 );

    snprintf( buf, sizeof( buf ), "Unexpected exception: 0x%02x PC:0x%06x\n", vectorNum, pc );

    while ( *s != '\0' )
    {
        qsm_polled_outbyte( *s++ );
    }

    while ( 1 )
    {
        ;
    }
}
                                   
//***************************************************************************
/**
*   Initializes all exception vectors to point to our exception handler
*/
                                   
void ExceptionInit( void )
{
    for ( int i = 2; i < 255; i++ ) 
    {
        install_isr( i, (void (*)())ExceptionHandler );
    }
}

