/****************************************************************************
*
*     Copyright (c) 2003 Dave Hylands
*           All Rights Reserved
*
*   Permission is granted to any individual or institution to use, copy, or
*   redistribute this software so long as it is not sold for profit, and that
*   this copyright notice is retained.
*
****************************************************************************/
/**
*
*   @file   Tachometer.h
*
*   @brief  Describes the interface to a tachometer.
*
****************************************************************************/

#if !defined( TACHOMETER_H )
#define TACHOMETER_H                   ///< Include Guard

// ---- Include Files -------------------------------------------------------

#include <68332/tpu.h>

/**
 * @addtogroup Peripheral
 * @{
 */

//---------------------------------------------------------------------------
/**
*   The Tachometer class encapsulates the TPU signals required to drive a motor.
*/

class Tachometer
{
public:

    //------------------------------------------------------------------------
    // Default constructor

    Tachometer( const char *name );

    //------------------------------------------------------------------------
    // Destructor

    ~Tachometer();

    //------------------------------------------------------------------------
    // Initialize the TPU pins

    void Init( int pin );

    //------------------------------------------------------------------------
    // Rpm - returns the tachometer reading in Revolutions per minute

    unsigned int Rpm();

    //------------------------------------------------------------------------
    // SetDivisionsPerRev - Sets the frequency used by the motor.

    void     SetDivisionsPerRev( unsigned divPerRev );
    unsigned GetDivisionsPerRev() { return m_divPerRev; }

private:

    //------------------------------------------------------------------------
    // The copy constructor and assignment operator are not need for this
    // class so we declare them private and don't provide an implementation.

    Tachometer( const Tachometer & copy );
    Tachometer &operator =( const Tachometer &rhs );

    //------------------------------------------------------------------------

    static void InterruptHandler( unsigned int vectorNum );

    //------------------------------------------------------------------------

    const char  *m_name;
    TPUChannel  *m_channel;
    unsigned     m_divPerRev;

    enum { HistoryLen = 12 };

    uint32       m_divAggr;
    uint32       m_divCount[ HistoryLen ];
    int          m_divOffset;
};

// ---- Inline Functions ----------------------------------------------------

/** @} */

#endif // TACHOMETER_H

